package foo;

import org.springframework.beans.factory.annotation.Autowired;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.beans.factory.support.AbstractAutowireCapableBeanFactory;
import org.springframework.context.ApplicationContextInitializer;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.stereotype.Service;
import org.springframework.boot.SpringApplication;

import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

import org.junit.Test;

/**
 * Unit test for simple App.
 */
@RunWith(SpringRunner.class)
@SpringBootTest(args = {"--spring.main.allow-circular-references=true"})
@ContextConfiguration( initializers = AppTest.CustomApplicationContextInitializer.class)
public class AppTest 
{

    @SpyBean
    private App.MyService service;

    @Test
    public void shouldAnswerWithTrue()
    {
        System.out.println("START TEST");
        System.out.println("service " + service);
        System.out.println("service.self " + service.getSelf());
        assertNotNull( service );
        assertNotNull( service.getSelf() );
        assertTrue( service.getClass().getName().contains("Mockito") );
    }

    static class CustomApplicationContextInitializer implements ApplicationContextInitializer<ConfigurableApplicationContext> {

        @Override
        public void initialize(ConfigurableApplicationContext configurableApplicationContext) {
            var factory = configurableApplicationContext.getBeanFactory();
            if (factory instanceof AbstractAutowireCapableBeanFactory) {
                ((AbstractAutowireCapableBeanFactory) factory).setAllowRawInjectionDespiteWrapping(true);
            }
        }
    }

    @SpringBootApplication
    static public class App {

        public static void main(String[] args) {
            SpringApplication.run(App.class, args);
        }

        @Service
        // This will be used with @SpyBean in the test
        public class MyService {

            @Autowired
            MyService self;

            public void svc() {self.transacted();}

            @Transactional
            public void transacted() {/*something important*/}

            /* just for debug */
            MyService getSelf() {return self;}
        }
    }
}
